import * as THREE from './threejs/three.module.min.js';
import Stats from './threejs/stats.module.min.js';
import { GLTFLoader } from './threejs/GLTFLoader.min.js';
import { OrbitControls } from './threejs/OrbitControls.min.js';
import { GUI } from './threejs/dat.gui.module.min.js';

/**********************************************************************
//
//  ALL GLOBAL VARIABLES DEFINED IN globals.js
//  WHICH IS INCLUDED IN THE HTML FILE
//  
//  ********  CONTAINER VARS  ********
//  containerDOM.getElementById('container');
//  width = containerDOM.clientWidth;
//  height = containerDOM.clientHeight;
//  ********  THREE.JS base objects  ********
//      camera, scene, renderer
//  ********  3D objects  ********
//      tetrahedronObj, textObj, cameraObj
//  ********  STATS  ****
//      stats, canvasWidth,   canvasHeight
//  ********  MODEL LOAD SWITCH  ********
//      modelLoaded
//  ********  Animate Switch  ********
//      var animateSwitch = 1;
//  ********  GUI Options  ********
//      xVelocity, yVelocity, zVelocity
//      xTetrahedronAngle, yTetrahedronAngle, zTetrahedronAngle
//      xTextAngle, yTextAngle, zTextAngle
//      stop: function(), start: function(), reset: function()
//
**********************************************************************/

init();
animate();

function init(){

  //Load GLB containing: tetrahedronObj, textObj, cameraObj
  loadObjects();

  //Add Stats Object
  stats = new Stats();
  containerDOM.appendChild( stats.dom );

  //Create Renderer
  renderer = new THREE.WebGLRenderer( { antialias: true, alpha: true } );
  renderer.setPixelRatio( window.devicePixelRatio );
  renderer.setSize(containerDOM.clientWidth , containerDOM.clientHeight );
  renderer.setClearColor(0x000000, 0); //Transparent Background (clear color)
  //Add renderer to Container
  containerDOM.appendChild( renderer.domElement );

  //Create scene, set BG color
  scene = new THREE.Scene();

  //Add Camera
  camera = new THREE.PerspectiveCamera( 20, containerDOM.clientHeight / containerDOM.clientWidth, 1, 10000 );
  camera.position.z = 64;

  //Add Lights
  addLights();

  //Add OrbitControls
  var orbit = new OrbitControls(camera, renderer.domElement);
  orbit.maxPolarAngle = Math.PI;// / 2;
  //Assign Global
  orbitCtrl = orbit;
  orbit.update();

  //Add Gui Panel
  var guiPanel = new GUI();
  var tetrahedronFolder = guiPanel.addFolder('Tetrahedron');
  var textFolder = guiPanel.addFolder('Text');
  tetrahedronFolder.add(options, 'xVelocity', -0.1, 0.1).name('X Velocity').listen();
  tetrahedronFolder.add(options, 'yVelocity', -0.1, 0.1).name('Y Velocity').listen();
  tetrahedronFolder.add(options, 'zVelocity', -0.1, 0.1).name('Z Velocity').listen();
  tetrahedronFolder.add(options, 'xTetrahedronAngle', 0, 360).name('X Angle').listen();
  tetrahedronFolder.add(options, 'yTetrahedronAngle', 0, 360).name('Y Angle').listen();
  tetrahedronFolder.add(options, 'zTetrahedronAngle', 0, 360).name('Z Angle').listen();
  textFolder.add(options, 'xTextVelocity', -0.1, 0.1).name('x Velocity').listen();
  textFolder.add(options, 'yTextVelocity', -0.1, 0.1).name('y Velocity').listen();
  textFolder.add(options, 'zTextVelocity', -0.1, 0.1).name('z Velocity').listen();
  textFolder.add(options, 'xTextAngle', 0, 360).name('x Angle').listen();
  textFolder.add(options, 'yTextAngle', 0, 360).name('y Angle').listen();
  textFolder.add(options, 'zTextAngle',  0, 360).name('z Angle').listen();
  guiPanel.add(options, 'start');
  guiPanel.add(options, 'stop');
  guiPanel.add(options, 'reset');
  guiPanel.add(options, 'resetCam');
  tetrahedronFolder.open();
  textFolder.open();

  //Add Event Listeners
  window.addEventListener( 'resize', onWindowResize );
  setGuiTop();
}

function onWindowResize() {
  width = containerDOM.clientWidth;
  height = containerDOM.clientHeight;
  camera.aspect = width / height;
  camera.updateProjectionMatrix();
  renderer.setSize( width, height );
  setGuiTop();
}

function setGuiTop() {
  var guiDiv = document.getElementsByClassName('dg ac');
  var canvasTop = document.querySelector('#container').getBoundingClientRect().top + window.scrollY;
  guiDiv[0].style.position = "fixed";
  guiDiv[0].style.top = canvasTop+'px';
}

var deg2rad = function(degree) { return degree*(Math.PI/180);};
var rad2deg = function(rad) { return ((rad * 180.0) / Math.PI);};

function loadObjects() {
  var loader1 = new GLTFLoader();
  loader1.load('https://carpentiertech.com/wp-content/plugins/ctech/cam+tet+c-v1.glb', handle_load1);
  var model;
  function handle_load1(gltf1){
    model = gltf1.scene;
    scene.add( model );
    modelLoaded = 1;
    //Traverse the scene
    model.traverse(function(obj){
        if(obj.name === 'tetrahedron'){
            tetrahedronObj = obj;
            tetrahedronObj.rotation.x = 0;
            tetrahedronObj.rotation.y = 0;
            tetrahedronObj.rotation.z = 0;
        }
        if(obj.name === 'Text'){
          textObj = obj;
          textObj.rotation.x = Math.PI / 2;
        }
        if(obj.isCamera){
          cameraObj = obj;
        }
    });
  }
}

function addLights() {
  //Add Lights
  const directionalLight0 = new THREE.DirectionalLight( 0xffffff, 2 );
  directionalLight0.position.set( 100, 100, 100 );
  directionalLight0.name = 'dl0';
  directionalLight0.lookAt(0,0,0);
  scene.add( directionalLight0 );

  //MANY LIGHTS
  const directionalLight1 = new THREE.DirectionalLight( 0xffffff, 2 );
  directionalLight1.position.set( -100, 100, 100 );
  directionalLight1.name = 'dl1';
  directionalLight1.lookAt(0,0,0);
  scene.add( directionalLight1 );
  const directionalLight2 = new THREE.DirectionalLight( 0xffffff, 3 );
  directionalLight2.position.set( 100, -100, 100 );
  directionalLight2.name = 'dl2';
  directionalLight2.lookAt(0,0,0);
  //scene.add( directionalLight2 );
  const directionalLight3 = new THREE.DirectionalLight( 0xffffff, 2 );
  directionalLight3.position.set( -100, -100, 100 );
  directionalLight3.name = 'dl3';
  directionalLight3.lookAt(0,0,0);
  scene.add( directionalLight3 );
  const directionalLight4 = new THREE.DirectionalLight( 0xffffff, 1 );
  directionalLight4.position.set( 0, 0, -100 );
  directionalLight4.name = 'dl4';
  //scene.add( directionalLight4 );
  const directionalLight5 = new THREE.DirectionalLight( 0xffffff, 1 );
  directionalLight5.position.set( 0, -100, 0 );
  directionalLight5.name = 'dl5';
  //scene.add( directionalLight5 );
  const directionalLight6 = new THREE.DirectionalLight( 0xffffff, 1 );
  directionalLight6.position.set( -100, 0, 0 );
  directionalLight6.name = 'dl6';
  //scene.add( directionalLight6 );

  const spotLight2 = new THREE.SpotLight( 0xffffff, 1 );
  spotLight2.position.set( 100, -100, 100 );
  spotLight2.castShadow = true;
  spotLight2.name = 'sl2';
  spotLight2.lookAt(0,0,0);
  scene.add( spotLight2 );

  const spotLight3 = new THREE.SpotLight( 0xffffff, 1 );
  spotLight3.position.set(-200, -200, -200 );
  spotLight3.angle = Math.PI / 9;
  spotLight3.castShadow = true;
  spotLight3.name = 'sl3';
  scene.add( spotLight3 );

  var ambientLight1 = new THREE.AmbientLight(0xffffff, 1 );
  ambientLight1.power = 13260;  // GE Lumens @ 120W incade.
  ambientLight1.decay = 2;
  ambientLight1.distance = Infinity;
  ambientLight1.position.set(0, 200, 0);
  ambientLight1.name = 'al1';
  //scene.add(ambientLight1);
  
  var ambientLight2 = new THREE.AmbientLight(0xffffff, 1 );
  ambientLight2.power = 13260;  // GE Lumens @ 120W incade.
  ambientLight2.distance = Infinity;
  ambientLight2.position.set(0, 0, 200);
  ambientLight2.name = 'al2';
  //scene.add(ambientLight2);  
}

function dump(obj) {
  var out = '';
  for (var i in obj) {
  out += i + ": " + obj[i] + "\n";
  }
 alert(out);
 console.log(out);
 }

function animate(){
  if (modelLoaded) {
		if ( options.xVelocity || options.yVelocity || options.zVelocity || options.xTextVelocity || options.yTextVelocity || options.zTextVelocity ) 
      { animateSwitch = 1; } else { animateSwitch = 0; }
	}

  if (modelLoaded && animateSwitch ){
    if(tetrahedronObj.rotation.x >= (Math.PI*2)){
      tetrahedronObj.rotation.x = tetrahedronObj.rotation.x - (Math.PI*2);
    }
    if(tetrahedronObj.rotation.x < 0 ){
      tetrahedronObj.rotation.x = tetrahedronObj.rotation.x + (Math.PI*2);
    }
    tetrahedronObj.rotation.x += options.xVelocity;

    if(tetrahedronObj.rotation.y >= (Math.PI*2)){
      tetrahedronObj.rotation.y = tetrahedronObj.rotation.y - (Math.PI*2);
    }
    if(tetrahedronObj.rotation.y < 0 ){
      tetrahedronObj.rotation.y = tetrahedronObj.rotation.y + (Math.PI*2);
    }      
    tetrahedronObj.rotation.y += options.yVelocity;

    if(tetrahedronObj.rotation.z >= (Math.PI*2)){
      tetrahedronObj.rotation.z = tetrahedronObj.rotation.z - (Math.PI*2);
    }
    if(tetrahedronObj.rotation.z < 0 ){
      tetrahedronObj.rotation.z = tetrahedronObj.rotation.z + (Math.PI*2);
    }
    tetrahedronObj.rotation.z += options.zVelocity;

    if(textObj.rotation.x >= (Math.PI*2)){
      textObj.rotation.x = textObj.rotation.x - (Math.PI*2);
    }
    if(textObj.rotation.x < 0 ){
      textObj.rotation.x = textObj.rotation.x + (Math.PI*2);
    }
    textObj.rotation.x += options.xTextVelocity;

    if(textObj.rotation.y >= (Math.PI*2)){
      textObj.rotation.y = textObj.rotation.y - (Math.PI*2);
    }
    if(textObj.rotation.y < 0 ){
      textObj.rotation.y = textObj.rotation.y + (Math.PI*2);
    }      
    textObj.rotation.y += options.yTextVelocity;

    if(textObj.rotation.z >= (Math.PI*2)){
      textObj.rotation.z = textObj.rotation.z - (Math.PI*2);
    }
    if(textObj.rotation.z < 0 ){
      textObj.rotation.z = textObj.rotation.z + (Math.PI*2);
    }
    textObj.rotation.z += options.zTextVelocity;

    options.xTetrahedronAngle = rad2deg((tetrahedronObj.rotation.x));
    options.zTetrahedronAngle = rad2deg(tetrahedronObj.rotation.z);
    options.yTetrahedronAngle = rad2deg(tetrahedronObj.rotation.y);
    options.xTextAngle = rad2deg(textObj.rotation.x);
    options.yTextAngle = rad2deg(textObj.rotation.z);
    options.zTextAngle = rad2deg(textObj.rotation.y);
  }
  
  if (modelLoaded && !animateSwitch ) {  
    tetrahedronObj.rotation.x = deg2rad(options.xTetrahedronAngle);
    tetrahedronObj.rotation.y = deg2rad(options.yTetrahedronAngle);
    tetrahedronObj.rotation.z = deg2rad(options.zTetrahedronAngle);
    textObj.rotation.x = deg2rad(options.xTextAngle);
    textObj.rotation.z = deg2rad(options.yTextAngle);
    textObj.rotation.y = deg2rad(options.zTextAngle);
  }
  
  requestAnimationFrame( animate );
  render();
  stats.update();
}

function render() {
  renderer.render( scene, camera );
}